import * as SQLite from "expo-sqlite";
import { assert, signalInvalidType } from "@canvas-js/utils";
import { AbstractModelDB, parseConfig, } from "@canvas-js/modeldb";
import { ModelAPI } from "./api.js";
export class ModelDB extends AbstractModelDB {
    db;
    #models = {};
    #transaction;
    constructor({ path, models, clear }) {
        super(parseConfig(models));
        this.db = SQLite.openDatabaseSync(path ?? ":memory:");
        if (clear) {
            this.db.withTransactionSync(() => {
                const tables = this.db.getAllSync("SELECT name FROM sqlite_master WHERE type='table'");
                for (const table of tables) {
                    // execSync works inconsistently in expo, use runSync instead
                    if (table.name.includes("\\") || table.name.includes('"')) {
                        throw new Error("unexpected table name, try clearing your database using Expo/Drizzle");
                    }
                    this.db.runSync(`DROP TABLE IF EXISTS "${table.name}"`);
                }
            });
        }
        for (const model of Object.values(this.models)) {
            this.#models[model.name] = new ModelAPI(this.db, model);
        }
        this.#transaction = (effects) => this.db.withTransactionSync(() => {
            for (const effect of effects) {
                const model = this.models[effect.model];
                assert(model !== undefined, `model ${effect.model} not found`);
                if (effect.operation === "set") {
                    this.#models[effect.model].set(effect.value);
                }
                else if (effect.operation === "delete") {
                    this.#models[effect.model].delete(effect.key);
                }
                else {
                    signalInvalidType(effect);
                }
            }
        });
    }
    getType() {
        return "sqlite-expo";
    }
    async close() {
        this.log("closing");
        this.db.closeSync();
    }
    async apply(effects) {
        this.#transaction(effects);
        for (const { model, query, filter, callback } of this.subscriptions.values()) {
            if (effects.some(filter)) {
                const api = this.#models[model];
                assert(api !== undefined, `model ${model} not found`);
                try {
                    callback(api.query(query));
                }
                catch (err) {
                    this.log.error(err);
                }
            }
        }
    }
    async get(modelName, key) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.get(key);
    }
    async getMany(modelName, keys) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.getMany(keys);
    }
    async *iterate(modelName, query = {}) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        yield* api.iterate(query);
    }
    async count(modelName, where) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.count(where);
    }
    async clear(modelName) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.clear();
    }
    async query(modelName, query = {}) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.query(query);
    }
}
